import { Options as RetryOptions } from 'p-retry';
import { DynamoDB } from '../dynamodb';
import { Table } from '../table';
import { Params } from '../types';
import { QueryBuilder } from '../types/query-builder';
export declare abstract class Method implements QueryBuilder {
    protected readonly table: Table | null;
    protected readonly dynamodb: DynamoDB;
    protected params: Params;
    protected retries?: number | RetryOptions;
    constructor(table: Table | null, dynamodb: DynamoDB);
    /**
     * Builds and returns the raw DynamoDB query object.
     */
    abstract buildRawQuery(): any;
    /**
     * Configure the number of retries.
     *
     * @param retries - Number of retries
     */
    retry(retries: number): any;
    /**
     * Configure the retry policy.
     *
     * @param retries - Retry configuration options.
     */
    retry(options: RetryOptions): any;
    protected runQuery<T>(operation: () => Promise<T>): Promise<T>;
}
