"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parse = exports.operators = void 0;
const set_1 = __importDefault(require("aws-sdk/lib/dynamodb/set"));
const nameUtil = __importStar(require("./name"));
exports.operators = ['$set', '$unset', '$inc', '$push', '$unshift', '$addToSet'];
const fromArrayEach = (value, allowArrayInArray = true) => {
    if (value.$each) {
        if (!Array.isArray(value.$each)) {
            throw new Error('The value for $each should be an array.');
        }
        return value.$each;
    }
    if (allowArrayInArray) {
        return [value];
    }
    return Array.isArray(value) ? value : [value];
};
function parse(query) {
    const names = {};
    const values = {};
    const expr = {};
    if (query.$set) {
        expr.set = expr.set || [];
        expr.set = expr.set.concat(Object.keys(query.$set).map(key => {
            const value = (query.$set)[key];
            const k = nameUtil.generateKeyName(key);
            const v = nameUtil.generateValueName(key, value, values, true);
            Object.assign(names, k.ExpressionAttributeNames);
            Object.assign(values, v.ExpressionAttributeValues);
            return k.Expression + '=' + v.Expression;
        }));
    }
    if (query.$unset) {
        expr.remove = expr.remove || [];
        expr.remove = expr.remove.concat(Object.keys(query.$unset).map(key => {
            const k = nameUtil.generateKeyName(key);
            Object.assign(names, k.ExpressionAttributeNames);
            return k.Expression;
        }));
    }
    if (query.$inc) {
        expr.set = expr.set || [];
        expr.set = expr.set.concat(Object.keys(query.$inc).map(key => {
            const value = (query.$inc)[key];
            const k = nameUtil.generateKeyName(key);
            const v = nameUtil.generateValueName(key, value, values);
            v.ExpressionAttributeValues[':_v_empty_value'] = 0;
            Object.assign(names, k.ExpressionAttributeNames);
            Object.assign(values, v.ExpressionAttributeValues);
            return `${k.Expression}=if_not_exists(${k.Expression}, :_v_empty_value)+${v.Expression}`;
        }));
    }
    if (query.$push || query.$unshift) {
        expr.set = expr.set || [];
        const operator = query.$push ? '$push' : '$unshift';
        expr.set = expr.set.concat(Object.keys(query[operator]).map(key => {
            const value = fromArrayEach((query[operator])[key]);
            const k = nameUtil.generateKeyName(key);
            const v = nameUtil.generateValueName(key, value, values, true);
            v.ExpressionAttributeValues[':_v_empty_list'] = [];
            Object.assign(names, k.ExpressionAttributeNames);
            Object.assign(values, v.ExpressionAttributeValues);
            // Use if_not_exists to solve failure if property does not exist https://github.com/SamVerschueren/dynongo/issues/29
            let args = `if_not_exists(${k.Expression}, :_v_empty_list), ${v.Expression}`;
            if (operator === '$unshift') {
                args = `${v.Expression}, if_not_exists(${k.Expression}, :_v_empty_list)`;
            }
            return k.Expression + '=list_append(' + args + ')';
        }));
    }
    if (query.$addToSet) {
        expr.add = expr.add || [];
        expr.add = expr.add.concat(Object.keys(query.$addToSet).map(key => {
            const value = fromArrayEach(query.$addToSet[key], false);
            const dynamoDBSet = new set_1.default(value);
            const k = nameUtil.generateKeyName(key);
            const v = nameUtil.generateValueName(key, dynamoDBSet, values, true);
            Object.assign(names, k.ExpressionAttributeNames);
            Object.assign(values, v.ExpressionAttributeValues);
            return `${k.Expression} ${v.Expression}`;
        }));
    }
    if (query.$removeFromSet) {
        expr.delete = expr.delete || [];
        expr.delete = expr.delete.concat(Object.keys(query.$removeFromSet).map(key => {
            const value = fromArrayEach(query.$removeFromSet[key], false);
            const dynamoDBSet = new set_1.default(value);
            const k = nameUtil.generateKeyName(key);
            const v = nameUtil.generateValueName(key, dynamoDBSet, values, true);
            Object.assign(names, k.ExpressionAttributeNames);
            Object.assign(values, v.ExpressionAttributeValues);
            return `${k.Expression} ${v.Expression}`;
        }));
    }
    const expression = Object.keys(expr).map(key => {
        if (expr[key].length === 0) {
            return undefined;
        }
        return key.toUpperCase() + ' ' + expr[key].join(', ');
    });
    return {
        UpdateExpression: expression.filter(Boolean).join(' '),
        ExpressionAttributeNames: names,
        ExpressionAttributeValues: values
    };
}
exports.parse = parse;
//# sourceMappingURL=update.js.map